/*
 * Created by Brett on 17/11/23.
 * Licensed under GNU General Public License V3.0
 * See LICENSE file for license detail
 */
#include <bf_tokenizer.h>
#include <sstream>
#include <random>
#include <iostream>

std::string generateName()
{
    std::stringstream ss;
    ss << std::hex;
    static std::random_device rd;
    static std::seed_seq seed{rd(), rd(), rd(), rd()};
    static std::mt19937_64 gen(seed);
    static std::uniform_int_distribution<int> dis(0, 15);
    
    for (int i = 0; i < 24; i++)
        ss << dis(gen);
    
    return ss.str();
}

void tokenizer::tokenize(const std::string& program)
{
    size_t index = 0;
    while (index < program.size())
    {
        auto c = program[index];
        switch (c)
        {
            case '>':
                tokens.emplace_back(bf_token::INC_DP);
                break;
            case '<':
                tokens.emplace_back(bf_token::DEC_DP);
                break;
            case '+':
                tokens.emplace_back(bf_token::INC_DV);
                break;
            case '-':
                tokens.emplace_back(bf_token::DEC_DV);
                break;
            case '.':
                tokens.emplace_back(bf_token::PRINT);
                break;
            case ',':
                tokens.emplace_back(bf_token::READ);
                break;
            case '[':
                tokens.emplace_back(bf_token::OPEN);
                break;
            case ']':
                tokens.emplace_back(bf_token::CLOSE);
                break;
            default:
                break;
        }
        index++;
    }
}

void tokenizer::bf_name()
{
    size_t search_index = 0;
    while (search_index < tokens.size())
    {
        if (tokens[search_index].token == bf_token::OPEN)
        {
            auto name = ++conditionalCount;
            size_t sp = 1;
            size_t search_2 = search_index;
            tokens[search_index].name = name;
            while (++search_2 < tokens.size())
            {
                if (tokens[search_2].token == bf_token::OPEN)
                    sp++;
                if (tokens[search_2].token == bf_token::CLOSE)
                    sp--;
                if (sp == 0)
                    break;
            }
            tokens[search_2].name = name;
        }
        search_index++;
    }
}

inline void tabulate(signed long long v)
{
    for (signed long long i = 0; i < v; i++)
        std::cout << '\t';
}

void tokenizer::print(size_t index)
{
    signed long long sp = 0;
    while (index < tokens.size())
    {
        auto& token = tokens[index++];
        switch (token.token)
        {
            case bf_token::INC_DP:
                tabulate(sp);
                std::cout << "Increase DP\n";
                break;
            case bf_token::DEC_DP:
                tabulate(sp);
                std::cout << "Decrease DP\n";
                break;
            case bf_token::INC_DV:
                tabulate(sp);
                std::cout << "Increase DV\n";
                break;
            case bf_token::DEC_DV:
                tabulate(sp);
                std::cout << "Decrease DV\n";
                break;
            case bf_token::PRINT:
                tabulate(sp);
                std::cout << "Print\n";
                break;
            case bf_token::READ:
                tabulate(sp);
                std::cout << "Read\n";
                break;
            case bf_token::OPEN:
                tabulate(sp);
                std::cout << "If(";
                if (token.name.has_value())
                    std::cout << token.name.value() << "\n";
                else
                    std::cout << "UNNAMED" << "\n";
                sp++;
                break;
            case bf_token::CLOSE:
                sp--;
                tabulate(sp);
                if (token.name.has_value())
                    std::cout << token.name.value();
                else
                    std::cout << "UNNAMED";
                std::cout << ")\n";
                break;
        }
    }
}

void tokenizer::print()
{
    print(currentIndex);
}
